### carbonquantPack.cmake --- 
######################################################################
## 
### Commentary: 
## 
######################################################################

include (InstallRequiredSystemLibraries)

## #################################################################
## Global settings
## #################################################################

set(CPACK_PACKAGE_NAME ${PROJECT_NAME})

set(CPACK_PACKAGE_VENDOR "Mireille Bossy")
set(CPACK_PACKAGE_CONTACT "Mireille Bossy <Mireille.Bossy@inria.fr>")
set(CPACK_PACKAGE_DESCRIPTION_FILE ${CMAKE_CURRENT_SOURCE_DIR}/README.txt)
set(CPACK_PACKAGE_DESCRIPTION_SUMMARY ${PROJECT_NAME})
set(CPACK_PACKAGE_DESCRIPTION ${PROJECT_NAME})
set(CPACK_PACKAGE_VERSION_MAJOR ${${PROJECT_NAME}_VERSION_MAJOR})
set(CPACK_PACKAGE_VERSION_MINOR ${${PROJECT_NAME}_VERSION_MINOR})
set(CPACK_PACKAGE_VERSION_PATCH ${${PROJECT_NAME}_VERSION_BUILD})
set(CPACK_RESOURCE_FILE_LICENSE ${CMAKE_CURRENT_SOURCE_DIR}/COPYING.txt)

set(CPACK_SOURCE_IGNORE_FILES 
  "~$"
  "/.svn/"
  "/.git/"
  "/.gitignore")

## #################################################################
## Generator list
## #################################################################

set(CPACK_BINARY_TBZ2 OFF CACHE BOOL "Enable to build TBZ2 packages")
set(CPACK_BINARY_TGZ OFF CACHE BOOL "Enable to build TGZ packages")
set(CPACK_BINARY_STGZ OFF CACHE BOOL "Enable to build STGZ packages")
set(CPACK_BINARY_TZ OFF CACHE BOOL "Enable to build TZ packages")
set(CPACK_BINARY_ZIP OFF CACHE BOOL "Enable to build ZIP packages")

## #################################################################
## CPack Configuration - Windows
## #################################################################

if (WIN32)
  set(CPACK_PACKAGE_FILE_NAME "${PROJECT_NAME}-${${PROJECT_NAME}_VERSION}.${CMAKE_SYSTEM_PROCESSOR}")
  set(CMAKE_INSTALL_PREFIX "C:\\\\Program Files\\\\${PROJECT_NAME}")
  set(CPACK_INSTALL_PREFIX ${CMAKE_INSTALL_PREFIX})
  set(CPACK_PACKAGING_INSTALL_PREFIX "")
  set(CPACK_BINARY_NSIS ON CACHE BOOL "Enable to build NSIS packages")
  set(CPACK_PACKAGE_EXECUTABLES "carbonquant" "CarbonQuant")
  set(CPACK_BUNDLE_ICON      "${PROJECT_SOURCE_DIR}\\\\app\\\\carbonquant\\\\icons\\\\carbonquant.ico")
  set(CPACK_PACKAGE_ICON "${PROJECT_SOURCE_DIR}\\\\app\\\\carbonquant\\\\icons\\\\carbonquant.ico")
  set(CPACK_CREATE_DESKTOP_LINKS "${PROJECT_NAME}" ${CPACK_CREATE_DESKTOP_LINKS})
  set(CPACK_NSIS_CONTACT "Mireille.Bossy@inria.fr")
  set(CPACK_NSIS_DISPLAY_NAME "CarbonQuant")
  set(CPACK_NSIS_HELP_LINK "http:\\\\\\\\carbonvalue.gforge.inria.fr")
  set(CPACK_NSIS_INSTALLED_ICON_NAME "bin\\\\carbonquant.exe")
  set(CPACK_NSIS_MENU_LINKS "http://carbonvalue.gforge.inria.fr" "CarbonQuant website")
  set(CPACK_NSIS_MODIFY_PATH ON)
  set(CPACK_NSIS_MUI_FINISHPAGE_RUN "carbonquant.exe")
  set(CPACK_NSIS_MUI_ICON     "${PROJECT_SOURCE_DIR}\\\\app\\\\carbonquant\\\\icons\\\\carbonquant.ico")
  set(CPACK_NSIS_MUI_UNIICON  "${PROJECT_SOURCE_DIR}\\\\app\\\\carbonquant\\\\icons\\\\carbonquant.ico")
  set(CPACK_NSIS_PACKAGE_NAME "CarbonQuant")
endif (WIN32)

## #################################################################
## CPack Configuration - Linux
## #################################################################

if (UNIX AND NOT APPLE)
  if(CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT)
    set(CMAKE_INSTALL_PREFIX "/usr/local/share" CACHE PATH "${PROJECT_NAME} install prefix" FORCE)
  endif(CMAKE_INSTALL_PREFIX_INITIALIZED_TO_DEFAULT)

  #set (CPACK_PACKAGE_ICON "${PROJECT_SOURCE_DIR}/app/carbonquant/carbonquant.ico")
  set (CPACK_RPM_PACKAGE_DEBUG 0)
  set (CPACK_BINARY_DEB "ON" CACHE BOOL "Enable to build Debian packages")

  execute_process(COMMAND lsb_release -irs
    COMMAND sed "s/ //"
    COMMAND sed "s/Fedora/fc/"
    COMMAND tr -d '\n'
    OUTPUT_VARIABLE DISTRIB
    OUTPUT_STRIP_TRAILING_WHITESPACE)
  execute_process(COMMAND arch 
    OUTPUT_VARIABLE ARCH
    OUTPUT_STRIP_TRAILING_WHITESPACE)

  if ((${CPACK_BINARY_DEB} MATCHES "ON") OR (${CPACK_BINARY_RPM} MATCHES "ON"))
    set(CPACK_PACKAGE_FILE_NAME "${PROJECT_NAME}-${${PROJECT_NAME}_VERSION}-${DISTRIB}-${ARCH}")
  else()
    set(CPACK_PACKAGE_FILE_NAME "${PROJECT_NAME}-${${PROJECT_NAME}_VERSION}")
  endif()

  set(CPACK_SOURCE_PACKAGE_FILE_NAME "${PROJECT_NAME}-${${PROJECT_NAME}_VERSION}-src")
  set(CPACK_PACKAGING_INSTALL_PREFIX "${CMAKE_INSTALL_PREFIX}/${PROJECT_NAME}" CACHE PATH "${PROJECT_NAME} packaging install prefix")

endif (UNIX AND NOT APPLE)

## #################################################################
## CPack Configuration - Mac
## #################################################################

if (APPLE)
   set (CMAKE_INSTALL_PREFIX "/opt/local")
   set (CPACK_INSTALL_PREFIX ${CMAKE_INSTALL_PREFIX})
   set (CPACK_GENERATOR "PackageMaker")
   set (CPACK_BUNDLE_NAME ${PROJECT_NAME})
   #set (CPACK_GENERATOR "DragNDrop")
   set (CPACK_NSIS_DISPLAY_NAME "${CPACK_PACKAGE_INSTALL_DIRECTORY} Carbonquant")
   # set (CPACK_NSIS_MODIFY_PATH ON) # Choice to add prog dir to the system PATH
   set (CPACK_PACKAGE_DEFAULT_LOCATION "/")
   set(CPACK_PACKAGE_FILE_NAME "${PROJECT_NAME}-${${PROJECT_NAME}_VERSION}.${CMAKE_SYSTEM_PROCESSOR}")
   set(CPACK_PACKAGING_INSTALL_PREFIX ${CMAKE_INSTALL_PREFIX} CACHE PATH "${PROJECT_NAME} packaging install prefix")
endif (APPLE)

## #################################################################
## 
## #################################################################

if (APPLE)
add_custom_command (OUTPUT ${CMAKE_BINARY_DIR}/.deploy
                   COMMAND ${CMAKE_BINARY_DIR}/modules/dtk-clone-enas-cq/bin/dtkDeploy ${CMAKE_BINARY_DIR}/bin/carbonquant.app -inject=${CMAKE_BINARY_DIR}/plugins
                   WORKING_DIRECTORY ${CMAKE_BINARY_DIR} )

add_custom_command (OUTPUT ${CMAKE_BINARY_DIR}/${CPACK_PACKAGE_FILE_NAME}.dmg 
                    COMMAND /usr/bin/hdiutil create -fs HFS+ -srcfolder ${CMAKE_BINARY_DIR}/bin/carbonquant.app/ -volname Carbonquant  ${CMAKE_BINARY_DIR}/${CPACK_PACKAGE_FILE_NAME}.dmg
                    WORKING_DIRECTORY ${CMAKE_BINARY_DIR} )

add_custom_command (OUTPUT ${CMAKE_BINARY_DIR}/.clean_dmg
                   COMMAND if test -f ${CMAKE_BINARY_DIR}/${CPACK_PACKAGE_FILE_NAME}.dmg \; then rm   ${CMAKE_BINARY_DIR}/${CPACK_PACKAGE_FILE_NAME}.dmg\; fi
                   WORKING_DIRECTORY ${CMAKE_BINARY_DIR} )
add_custom_target (clean_dmg
 		    DEPENDS ${CMAKE_BINARY_DIR}/.clean_dmg    
                    COMMENT "Cleaning dmg file(s)")

add_custom_target (dmg
 		    DEPENDS ${CMAKE_BINARY_DIR}/${CPACK_PACKAGE_FILE_NAME}.dmg  ${CMAKE_BINARY_DIR}/.deploy  
                    COMMENT "update the packages")
endif (APPLE)

## #################################################################
## Install prefix
## #################################################################

mark_as_advanced(CPACK_PACKAGING_INSTALL_PREFIX)

## #################################################################
## NSIS generator settings
## #################################################################

# ;

## #################################################################
## RPM generator settings
## #################################################################

# ;

## #################################################################
## DEB generator settings
## #################################################################

set(CPACK_DEBIAN_PACKAGE_DEPENDS "libc6 (>= 2.4), libgcc1 (>= 1:4.1.1), libqt4-opengl (>= 4:4.5.3), libqtcore4 (>= 4:4.6.1), libqtgui4 (>= 4:4.6.1), libstdc++6 (>= 4.4.0)")

## #################################################################
## Enable packaging
## #################################################################

include(CPack)
