#ifndef carbonquant_CalibrationEXP_OU_VG_h
#define carbonquant_CalibrationEXP_OU_VG_h

#include "Calibration.h"

class CARBONQUANTCALIBRATIONMODELSPOT_EXPORT CalibrationEXP_OU_VG : public Calibration
{
protected:
  const int n_sum;

  double meanReversion;
  double meanParameter;

  std::vector<double> moments_obs;
  std::vector<std::vector<double> > Cnp_mat;

public: 
  CalibrationEXP_OU_VG();
  virtual ~CalibrationEXP_OU_VG();

  virtual void init();

  // Init parameters from spot
  virtual void initParametersFromSpot(const Spot* spot, const std::map<std::string, double>& spot_bound, const std::vector<double>& spotObs, const double& timeStep, const double& spotScale);

  // Set spot from best model parameters
  virtual void initSpotFromParameters(Spot *spot);

  // function to compute the objective function 
  virtual std::vector<double> f(const std::vector<double>& point) const;

  // calibrated model
  double cdf_model(const double& x, const int& n, const std::vector<double>& param) const;
  virtual std::vector<double> cdf_model(const double& born_min, const double& born_max, const int& size_of_cdf_obs) const;
  virtual std::vector<double> cdf_model_day_night(const double& born_min, const double& born_max, const int& size_of_cdf_obs, const std::vector<double>& param_day, const std::vector<double>& param_night) const;

  virtual double delta_rentability_obs(const double& rentabilityObs, const double& rentabilityObsAfter, const std::vector<double>& param) const;
  virtual double rentability_model(const double& rentabilityModelBefore, const std::vector<double>& param) const;

  // Compute calibration
  virtual void compute(Optimizer* solver);

  void compute_Cnp_mat(int);
  void compute_moments_obs(int);

  // Return model name
  virtual std::string name_of_model() const;  
};

#endif // carbonquant_CalibrationEXP_OU_VG_h
 
