/* carbonquantComposerTab.h --- 
 */

/* Commentary: 
 * 
 */

#ifndef CARBONQUANTCOMPOSERTAB_H
#define CARBONQUANTCOMPOSERTAB_H

#include "carbonquantGuiExport.h"

#include <dtkComposer/dtkComposerWriter.h>

#include <QtGui/QFrame>

class carbonquantSpotTab;
class carbonquantPortfolioTab;
class carbonquantPDFTab;
class carbonquantHJBTab;

class carbonquantComposerTabPrivate;

class dtkComposerSceneNodeLeaf;
class dtkAbstractView;

class CARBONQUANTGUI_EXPORT carbonquantComposerTab : public QFrame
{
    Q_OBJECT

public:
    carbonquantComposerTab(QWidget *parent = 0);
    ~carbonquantComposerTab(void);

    enum CompositionType {
        DataOnly,
        Spot,
        SpotDayNight,
        Calibration,
        CalibrationDayNight,
        SpotGraphs,
        ControlPath,
        ControlPathGraphs,
        PDF,
        PDFGraphs,
        HJB,
        HJBIndifferencePrice,
        HJBIsolines,
        HJBGraphs,
        Graphs
    };

    bool isComposerModified(void);
    bool open(const QString&);
    bool compositionOpen(const QString&);
    bool save(const QString&, dtkComposerWriter::Type);
    bool compositionSave(const QString&, dtkComposerWriter::Type, CompositionType compositionType = DataOnly);

    void setSpotTab(carbonquantSpotTab*);
    void setPortfolioTab(carbonquantPortfolioTab*);
    void setPDFTab(carbonquantPDFTab*);
    void setHJBTab(carbonquantHJBTab*);

    void compositionAddGraphs(const QString&, CompositionType);

    dtkComposerSceneNode* getSceneNode(QString label) const;

protected:
    void mouseMoveEvent(QMouseEvent*) {};
    void mousePressEvent(QMouseEvent*) {};
    void mouseReleaseEvent(QMouseEvent*) {};

    QString createXmlData(QString parent = "root");

    void updateComposition(CompositionType compositionType = DataOnly);

    void runProcess(CompositionType, QString processTitle = QString());

    void updateSpotTab(void);
    void updatePortfolioTab(void);
    void updatePDFTab(void);
    void updateHJBTab(void);

    void addDataOnlyToComposition(uint&);
    void addReferencePeriodDataToComposition(uint&, int&, int&);
    void addPowerArrayDataToComposition(uint&, int&, int&);
    void addDataClassToComposition(uint&, int&, int&);
    void addSpotDataToComposition(uint&, int&, int&, QString type = QString());
    void addSpotDayNightDataToComposition(uint&, int&, int&);
    void addCalibrationDataToComposition(uint&, int&, int&);
    void addCalibrationDayNightDataToComposition(uint&, int&, int&);
    void addPortfolioDataToComposition(uint&, int&, int&);
    void addControlPathDataToComposition(uint&, int&, int&);
    void addPDFDataToComposition(uint&, int&, int&);
    void addHJBDataToComposition(uint&, int&, int&);
    void addHJBIsolinesDataToComposition(uint&, int&, int&);

    void addSpotGraphsDataToComposition(uint&, int&, int&);
    void addControlPathGraphsDataToComposition(uint&, int&, int&);
    void addPDFGraphsDataToComposition(uint&, int&, int&);
    void addHJBGraphsDataToComposition(uint&, int&, int&);
    void addGraphsDataToComposition(uint&, int&, int&, QList<const dtkAbstractView*>);

    void addReferencePeriodToComposition(uint&, int&, int&);
    void addSpotToComposition(uint&, int& x, int&, QString type = QString());
    void addSpotPlotToComposition(uint&, int&, int&);
    void addSpotDayNightToComposition(uint&);
    void addSimulateSpotToComposition(uint&, QString type = QString());
    void addCalibrationToComposition(uint&);
    void addCalibrationPlotToComposition(uint&, int&, int&);
    void addCalibrationDayNightToComposition(uint&);
    void addControlPathToComposition(uint&);
    void addPDFToComposition(uint&);
    void addHJBToComposition(uint&);
    void addHJBIndifferencePriceToComposition(uint&);
    void addHJBIndifferencePricePlotToComposition(uint&, int&, int&, QString parent = "root", QString name = QString());
    void addHJBIsolinesToComposition(uint&);
    void addHJBDiscreteSchemeToComposition(uint&, int&, int&, const QStringList&, const QString&, QString parent = "root");
    void addHJBTimeDiscretisationToComposition(uint&, int&, int&, const QStringList&, const QString&);
    void addHJBPolicyIterationToComposition(uint&, int&, int&, const QStringList&, const QString&, QString parent = "root");

signals:
    void updatePlotTitles(void);

public slots:
    void setModified(bool);

    void onSpotClicked(void);
    void onSpotDayNightClicked(void);
    void onCalibrationClicked(void);
    void onCalibrationDayNightClicked(void);
    void onControlPathClicked(void);
    void onPDFClicked(void);
    void onHJBClicked(void);
    void onHJBIndifferencePriceClicked(void);
    void onHJBIsolinesClicked(void);

protected slots:
    void switchToCompo(void);
    void switchToDstrb(void);
    //void switchToDebug(void);

private:
    carbonquantComposerTabPrivate *d;
};

#endif
