/* carbonquantHJBTab.h --- 
 */

/* Commentary: 
 * 
 */

#ifndef CARBONQUANTHJBTAB_H
#define CARBONQUANTHJBTAB_H

#include "carbonquantGuiExport.h"

#include <QtGui/QFrame>

class carbonquantSpotTab;
class carbonquantComposerTab;
class carbonquantHJBView;

class carbonquantHJBTabPrivate;

class CARBONQUANTGUI_EXPORT carbonquantHJBTab : public QFrame
{
    Q_OBJECT

public:
    enum Scheme {
        Implicit,
        Explicit
    };

    enum LinearSolver {
        BGSTAB,
        GMRES,
        CG,
        Jacobi,
        GaussSeidel
    };

public:
     carbonquantHJBTab(QWidget *parent = 0);
    ~carbonquantHJBTab(void);

    void setSpotTab(carbonquantSpotTab*);
    void setComposerTab(carbonquantComposerTab*);

    const carbonquantHJBView* hjbView(void) const;

    void importData(const QString&);

    void setTheta(const qreal&);
    void setCFL(const qreal&);
    void setNumberOfTimeStepsImposed(const int&);
    void setMaximumNumberOfTimeSteps(const qlonglong&);
    void setTolerance(const qreal&);
    void setNumberOfPolicyIterations(const int&);
    void setLinearSolver(const int&);
    void setGridDimension(const int&);
    void setEmissionMinValue(const qreal&);
    void setEmissionMaxValue(const qreal&);
    void setEmissionNumberOfValues(const int&);
    void setSpotMinValue(const qreal&);
    void setSpotMaxValue(const qreal&);
    void setSpotNumberOfValues(const int&);
    void setViewRefreshTimeStep(const int&);
    void setWriteValueFunctionHJBFile(const int&);
    void setValueFunctionHJBFileName(const QString&);
    void setAllowedSize(const qreal&);
    void setIsolinesGraph(const QString&);
    void setIsolinesXaxis(const int&);
    void setIsolinesYaxis(const int&);
    void setIsolinesZvalues(const QString&);

    qreal theta(void);
    qreal cfl(void);
    int numberOfTimeStepsImposed(void) const;
    qlonglong maximumNumberOfTimeSteps(void);
    qreal tolerance(void);
    int numberOfPolicyIterations(void);
    int linearSolver(void);
    int gridDimension(void);
    qreal emissionMinValue(void);
    qreal emissionMaxValue(void);
    int emissionNumberOfValues(void);
    qreal spotMinValue(void);
    qreal spotMaxValue(void);
    int spotNumberOfValues(void);
    int viewRefreshTimeStep(void);
    int writeValueFunctionHJBFile(void) const;
    QString valueFunctionHJBFileName(void) const;
    qreal allowedSize(void);
    QString isolinesGraph(void);
    int isolinesXaxis(void);
    int isolinesYaxis(void);
    QString isolinesZvalues(void);

    void clean(void);

protected:
    bool checkData(void);
    bool checkView(void);

    void exportGraphsData(const QString&);

signals:
    void hjbClicked(void);
    void hjbIndifferencePriceClicked(void);
    void hjbIsolinesClicked(void);

protected slots:
    void onSchemeChanged(const int&);
    void onNumberOfTimeStepsImposedClicked(int);
    void updateDefaultMaximumNumberOfTimeSteps(void);

    void onGridDimensionChanged(const int&);

    void onWriteValueFunctionHJBFileClicked(int);
    void onValueFunctionHJBFile(void);

    void onHJBRunClicked(void);
    void onHJBIndifferencePriceRunClicked(void);
    void onHJBIsolinesRunClicked(void);

    void onShowAreaData(bool);

    void addGraphsData(void);
    void exportGraphsData(void);
    void addGraphsData(const QString&);

private:
    carbonquantHJBTabPrivate *d;
};

#endif
