/* carbonquantSpotTab.h --- 
 */

/* Commentary: 
 * 
 */

#ifndef CARBONQUANTSPOTTAB_H
#define CARBONQUANTSPOTTAB_H

#include "carbonquantGuiExport.h"

#include <carbonquantConfig.h>

#include <QtGui/QFrame>

class carbonquantComposerTab;
class carbonquantSpotView;
class QDate;

class carbonquantSpotTabPrivate;

class CARBONQUANTGUI_EXPORT carbonquantSpotTab : public QFrame
{
    Q_OBJECT

public:
    enum SpotType {
        Simple,
        DayNight
    };

public:
     carbonquantSpotTab(QWidget *parent = 0);
    ~carbonquantSpotTab(void);

    void setComposerTab(carbonquantComposerTab*);

    const carbonquantSpotView* spotView(void) const;

    int spotModel(void) const;
    SpotType spotType(void) const;
    int calibrationState(void) const;
    QString epexSpotFileName(void) const;
    QString spotFileName(void) const;
    int spotScaleType(void) const;
    QString startDate(void) const;
    QString startHour(void) const;
    QString endDate(void) const;
    QString endHour(void) const;
    QString unitOfTime(void) const;
    double simulationTimeStep(void) const;

    int startHourDay(void) const;
    int endHourDay(void) const;
    int startHourNight(void) const;
    int endHourNight(void) const;

    double initialSpotPrice(void) const;
    double spotScale(void) const;
    double volatility(void) const;
    double meanParameter(void) const;
    double meanReversion(void) const;
    double vgAdditionalDriftM(void) const;
    double vgGammaShapeKappa(void) const;
    double vgBrownianDriftCoefTheta(void) const;

    double initialSpotPriceNight(void) const;
    double spotScaleNight(void) const;
    double volatilityNight(void) const;
    double meanParameterNight(void) const;
    double meanReversionNight(void) const;
    double vgAdditionalDriftMNight(void) const;
    double vgGammaShapeKappaNight(void) const;
    double vgBrownianDriftCoefThetaNight(void) const;

    double initialSpotPriceDayNight(void) const;

    double calibrationInitialVolatility(void) const;
    double calibrationMinVolatility(void) const;
    double calibrationMaxVolatility(void) const;
    double calibrationInitialMeanParameter(void) const;
    double calibrationMinMeanParameter(void) const;
    double calibrationMaxMeanParameter(void) const;
    double calibrationInitialMeanReversion(void) const;
    double calibrationMinMeanReversion(void) const;
    double calibrationMaxMeanReversion(void) const;
    double calibrationInitialVgAdditionalDriftM(void) const;
    double calibrationMinVgAdditionalDriftM(void) const;
    double calibrationMaxVgAdditionalDriftM(void) const;
    double calibrationInitialVgGammaShapeKappa(void) const;
    double calibrationMinVgGammaShapeKappa(void) const;
    double calibrationMaxVgGammaShapeKappa(void) const;
    double calibrationInitialVgBrownianDriftCoefTheta(void) const;
    double calibrationMinVgBrownianDriftCoefTheta(void) const;
    double calibrationMaxVgBrownianDriftCoefTheta(void) const;

    int optimizerType(void) const;
    double optimizerTolerance(void) const;
    int optimizerMaxNumberOfIte(void) const;
    double optimizerStep(void) const;
    int optimizerPopulationSize(void) const;

    void setSpotType(SpotType);
    void setCalibrationState(const int&);

    void setEpexSpotFileName(const QString&);
    void setSpotModel(const int&);
    void setSpotScaleType(const int&);
    void setStartDate(const QString&);
    void setStartHour(const QString&);
    void setEndDate(const QString&);
    void setEndHour(const QString&);
    void setUnitOfTime(const QString&);
    void setSimulationTimeStep(const double&);

    void setStartHourDay(const int&);
    void setEndHourDay(const int&);
    void setStartHourNight(const int&);
    void setEndHourNight(const int&);

    void setInitialSpotPrice(const double&);
    void setSpotScale(const double&);
    void setVolatility(const double&);
    void setMeanParameter(const double&);
    void setMeanReversion(const double&);
    void setVgAdditionalDriftM(const double&);
    void setVgGammaShapeKappa(const double&);
    void setVgBrownianDriftCoefTheta(const double&);

    void setInitialSpotPriceNight(const double&);
    void setSpotScaleNight(const double&);
    void setVolatilityNight(const double&);
    void setMeanParameterNight(const double&);
    void setMeanReversionNight(const double&);
    void setVgAdditionalDriftMNight(const double&);
    void setVgGammaShapeKappaNight(const double&);
    void setVgBrownianDriftCoefThetaNight(const double&);

    void setCalibrationInitialVolatility(const double&);
    void setCalibrationMinVolatility(const double&);
    void setCalibrationMaxVolatility(const double&);
    void setCalibrationInitialMeanParameter(const double&);
    void setCalibrationMinMeanParameter(const double&);
    void setCalibrationMaxMeanParameter(const double&);
    void setCalibrationInitialMeanReversion(const double&);
    void setCalibrationMinMeanReversion(const double&);
    void setCalibrationMaxMeanReversion(const double&);
    void setCalibrationInitialVgAdditionalDriftM(const double&);
    void setCalibrationMinVgAdditionalDriftM(const double&);
    void setCalibrationMaxVgAdditionalDriftM(const double&);
    void setCalibrationInitialVgGammaShapeKappa(const double&);
    void setCalibrationMinVgGammaShapeKappa(const double&);
    void setCalibrationMaxVgGammaShapeKappa(const double&);
    void setCalibrationInitialVgBrownianDriftCoefTheta(const double&);
    void setCalibrationMinVgBrownianDriftCoefTheta(const double&);
    void setCalibrationMaxVgBrownianDriftCoefTheta(const double&);

    void setOptimizerType(const int&);
    void setOptimizerTolerance(const double&);
    void setOptimizerMaxNumberOfIte(const int&);
    void setOptimizerStep(const double&);
    void setOptimizerPopulationSize(const int&);

    bool writeSpotFileFromEpexSpotFile(void);

    bool writeSpotViewPrices(const QString&);
    bool writePrices(const QString&, const QList<double>&);

    void clean(void);

protected:
    void exportGraphsData(const QString&);

signals:
    void spotClicked(void);
    void spotDayNightClicked(void);
    void calibrationClicked(void);
    void calibrationDayNightClicked(void);

    void onStartDateChanged(const QDate&);
    void onStartHourChanged(const QTime&);
    void onEndDateChanged(const QDate&);
    void onEndHourChanged(const QTime&);
    void onUnitOfTimeChanged(const QString&);
    void onSimulationTimeStepChanged(const QString&);

public slots:
    void readComposerSpotParameters(void);

protected slots:
    void onModelChanged(const QString&);
    void onTypeChanged(const QString&);
    void onOptimizerTypeChanged(const QString&);

    void onCalibrationStateClicked(int);
    void onSpotFile(void);
    void onCalibrationRunClicked(void);

    void onSpotRunClicked(void);

    void onShowAreaData(bool);

    void addGraphsData(void);
    void exportGraphsData(void);
    void addGraphsData(const QString&);

private:
    carbonquantSpotTabPrivate *d;
};

#endif
