/* carbonquantStructuredGrid.h --- 
 */

/* Commentary: 
 * 
 */

#ifndef CARBONQUANTSTRUCTUREDGRID_H
#define CARBONQUANTSTRUCTUREDGRID_H

#include "carbonquantHJBExport.h"

#include <dtkCore/dtkAbstractData.h>

template <typename T> class dtkContainerVector;

class carbonquantStructuredGridPrivate;

// /////////////////////////////////////////////////////////////////
// carbonquantStructuredGrid interface
// /////////////////////////////////////////////////////////////////

class CARBONQUANTHJB_EXPORT carbonquantStructuredGrid : public dtkAbstractData
{
    Q_OBJECT

public:
    enum IndexesType {
        All,
        Inside,
        AllBoundaries,
        Boundary
    };

public:
    carbonquantStructuredGrid(void);
    carbonquantStructuredGrid(const carbonquantStructuredGrid& other);
    ~carbonquantStructuredGrid(void);
    
public:
    carbonquantStructuredGrid& operator = (const carbonquantStructuredGrid& other);
    
    bool operator == (const carbonquantStructuredGrid& other) const;
    
public:
    QString description(void) const;
    QString  identifier(void) const;

public:
    void clear(void);

    void setNumberOfDimensions(int number_of_dimension);
    
    void setDimensionName(int dimension_index, const QString& name);

    void setNumberOfPoints(int dimension_index, qlonglong number_of_points);

    void setCoordinate(int dimension_index, qlonglong point_index, qreal coordinate);

    void setCoordinates(int dimension_index, const dtkContainerVector<qreal>& coordinates);

public:
    const int& numberOfDimensions(void) const;

    const qlonglong& numberOfPoints(int dimension_index) const;

    qlonglong totalNumberOfPoints(void) const;

    const QString& dimensionName(int dimension_index) const;

    const qreal& coordinate(int dimension_index, qlonglong point_index) const;

    const dtkContainerVector<qreal>& coordinates(int dimension_index) const;

    qlonglong pointId(const QVector<qlonglong>&) const;

    dtkContainerVector<QVector<qlonglong> > indexes(IndexesType indexes_type = All, int dimension_index = -1, qlonglong point_index = -1) const;

protected:
    dtkContainerVector<QVector<qlonglong> > searchIndexes(int coefficient = 0, int dimension_index = -1, qlonglong point_index = -1) const;

private:
    DTK_DECLARE_PRIVATE(carbonquantStructuredGrid);
};

// /////////////////////////////////////////////////////////////////
// Debug operators
// /////////////////////////////////////////////////////////////////

CARBONQUANTHJB_EXPORT QDebug operator<<(QDebug dbg, carbonquantStructuredGrid& grid);
CARBONQUANTHJB_EXPORT QDebug operator<<(QDebug dbg, carbonquantStructuredGrid *grid);

// /////////////////////////////////////////////////////////////////
// Instanciation function
// /////////////////////////////////////////////////////////////////

CARBONQUANTHJB_EXPORT dtkAbstractData *createCarbonquantStructuredGrid(void);

// /////////////////////////////////////////////////////////////////
//
// /////////////////////////////////////////////////////////////////

Q_DECLARE_METATYPE(carbonquantStructuredGrid);
Q_DECLARE_METATYPE(carbonquantStructuredGrid *);

#endif
