#ifndef carbonquant_SpotBM_h
#define carbonquant_SpotBM_h

#include "Spot.h"

/** Brownian motion with drift
*/
class CARBONQUANTMODELSPOT_EXPORT SpotBM : public Spot 
{
protected:
    double coeffDrift; 
    double volatility;
    
public:
     SpotBM(double sigma, double mu);
    ~SpotBM(void);

// Copy
public:
    SpotBM(const SpotBM& spot);
    SpotBM& operator = (const SpotBM& spot);

public:
    virtual SpotBM* clone(void) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotBM"; }

    virtual Model get_model(void) const { return BM; }
  
    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //---------------------------------------------------------------------------
    virtual void set_volatility(const double& volatility) { this->volatility = volatility; };
    virtual void set_coeffDrift(const double& coeffDrift) { this->coeffDrift = coeffDrift; }; 

    //---------------------------------------------------------------------------
    virtual double get_volatility() const { return volatility; };
    virtual double get_coeffDrift() const { return coeffDrift; };

    //------------------------------------------------------------------------------------------------------
    virtual double simulate(const double &dt, const double &spotValue) const;
};

#endif // carbonquant_SpotBM_h
