#ifndef carbonquant_SpotBlackScholes_h
#define carbonquant_SpotBlackScholes_h

#include "Spot.h"

/** 

Black-Scholes Spot

**/
class CARBONQUANTMODELSPOT_EXPORT SpotBlackScholes : public Spot 
{
private:
    double volatility;      // volatility   (EDS : sigma)
    double rentability;     // rentability  (EDS : mu)

public:
     SpotBlackScholes(double sigma, double mu);
    ~SpotBlackScholes(void);

// Copy
public:
    SpotBlackScholes(const SpotBlackScholes& spot);
    SpotBlackScholes& operator = (const SpotBlackScholes& spot);

public:
    virtual SpotBlackScholes* clone(void) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotBlackScholes"; }
  
    virtual Model get_model(void) const { return BLACKSCHOLES; }

    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //---------------------------------------------------------------------------
    virtual void set_volatility(const double& volatility) { this->volatility = volatility; };
    virtual void set_rentability(const double& rentability) { this->rentability = rentability; }; 

    //---------------------------------------------------------------------------
    virtual double get_volatility() const { return volatility; };
    virtual double get_rentability() const { return rentability; };

    //-----------  function to simulate a spot ----------------------------------
    virtual double simulate(const double &dt, const double &spotValue) const;
};

#endif     // carbonquant_SpotBlackScholes_h
