#ifndef carbonquant_SpotDayNight_h
#define carbonquant_SpotDayNight_h

#include "Spot.h"

/** A simple Electricity Spot Class for day and night period
*/
class CARBONQUANTMODELSPOT_EXPORT SpotDayNight : public Spot
{  
    Spot *spot_day;
    Spot *spot_night;
  
    // definition of the period
    // day period   --- >  [begin_day,begin_night[
    // night period --- >  [begin_night,begin_day[
    int begin_day;
    int begin_night;

    // Scale hour
    double scale_hour;

    // begin hour (for simulation)
    double begin_hour;

public:
    SpotDayNight(Spot *s_day, Spot *s_night, const int & b_day, const int & b_night);
    virtual ~SpotDayNight();

    virtual Model get_model(void) const {return spot_day->get_model();};

    void set_spot_day(Spot *spot_day) {this->spot_day = spot_day;};
    void set_spot_night(Spot *spot_night) {this->spot_night = spot_night;};

    void set_begin_day(const int& begin_day) {this->begin_day = begin_day;};
    void set_begin_night(const int& begin_night) {this->begin_night = begin_night;};

    const Spot* get_spot_day() const {return spot_day;};
    const Spot* get_spot_night() const {return spot_night;};

    const int& get_begin_day() const {return begin_day;};
    const int& get_begin_night() const {return begin_night;};

    // t must be in hour
    bool determinate_period(double t) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotDayNight"; }

    //
    void setState(const std::vector<double>& state);
  
    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //-----------  function to simulate a spot --------------------------
    void set_begin_hour(double begin) {begin_hour = begin;};
    void set_scale_hour(double scale) {scale_hour = scale;};

    double simulate(const double &time_step, const double& time, const double &spotValue) const;
    virtual void simulate(const double &time_step, std::vector<double>& spotData) const;
};

#endif // carbonquant_SpotDayNight_h

