#ifndef carbonquant_SpotExponential_CIR_h
#define carbonquant_SpotExponential_CIR_h

#include "SpotOU.h"

// Function Added By El Hadj Oct 2011
/**** Exponential of a CIR Process *****/
class CARBONQUANTMODELSPOT_EXPORT SpotExponentialCIR : public Spot 
{
protected:  
    double meanParameter;  // mu 
    double meanReversion;  // alpha 
    double volatility;     // sigma 
    double s_const;
    double a_const; // default :: -1

public:
     SpotExponentialCIR(double sconst=1, double aconst=-1) { s_const = sconst; a_const = aconst;};
     SpotExponentialCIR(double alpha, double sigma, double mu, double sconst=1, double aconst=-1);
    ~SpotExponentialCIR(void);

// Copy
public:
    SpotExponentialCIR(const SpotExponentialCIR& spot);
    SpotExponentialCIR& operator = (const SpotExponentialCIR& spot);

public:
    virtual SpotExponentialCIR* clone(void) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotExponentialCIR"; }
  
    virtual Model get_model(void) const { return EXP_CIR; }

    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //--------------------------------------------------------------------------
    virtual void set_volatility(const double& volatility) { this->volatility = volatility; };
    virtual void set_meanParameter(const double& meanParameter) { this->meanParameter = meanParameter; }; 
    virtual void set_meanReversion(const double& meanReversion) { this->meanReversion = meanReversion; };
    virtual void set_spot_scale(const double& spot_scale) { this->s_const = spot_scale; };

    //---------------------------------------------------------------------------
    virtual double get_volatility() const { return volatility; };
    virtual double get_meanParameter() const { return meanParameter; }; 
    virtual double get_meanReversion() const { return meanReversion; };
    virtual double get_spot_scale() const { return s_const;};

    //-----------  function to simulate a spot ----------------------------------
    // function two simulate spot between two time
    virtual double simulate(const double &dt, const double &spotValue) const;

    virtual void print_model_param() const;
};

#endif // carbonquant_SpotExponential_CIR_h
