#ifndef carbonquant_SpotExponential_OU_h
#define carbonquant_SpotExponential_OU_h

#include "SpotOU.h"

// hypothesis :: x

// Exponential(OU process)
//
// S_t = s_const * exp(X_t) where X_t is an OU process
//

class CARBONQUANTMODELSPOT_EXPORT SpotExponentialOU : public SpotOU 
{
protected:
    double s_const; // see definition of the exp(OU) process

public:
     SpotExponentialOU(double scal=1) { s_const = scal; };
     SpotExponentialOU(double alpha, double sigma, double mu, double scal=1);
    ~SpotExponentialOU(void);

// Copy
public:
    SpotExponentialOU(const SpotExponentialOU& spot);
    SpotExponentialOU& operator = (const SpotExponentialOU& spot);

public:
    virtual SpotExponentialOU* clone(void) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotExponentialOU"; }

    virtual Model get_model(void) const { return EXP_OU; }
  
    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //--------------------------------------------------------------------------
    virtual void set_spot_scale(const double& spot_scale) { this->s_const = spot_scale; };

    //--------------------------------------------------------------------------
    virtual double get_spot_scale() const { return s_const;};

    //-----------  function to simulate a spot ----------------------------------
    // function two simulate spot between two time
    virtual double simulate(const double &dt, const double &spotValue) const;

    virtual void print_model_param() const;
};


#endif // carbonquant_SpotExponential_OU_h
