#ifndef carbonquant_SpotExponentialOUVG_h
#define carbonquant_SpotExponentialOUVG_h

#include "SpotOU.h"

/**** Exponential of an OU-VG Process *****/

//
// S_t = s_const *(  exp( X_t ) + a_const ) where X_t est un OU-VG process
//

class CARBONQUANTMODELSPOT_EXPORT SpotExponentialOUVG : public SpotOU {

protected:
    double s_const;
    double vgAdditionalDrift_m;       // m_0;
    double vgBrownianDriftCoef_theta; // theta_0;
    double vgGammaShape_kappa;        // kappa_0;
    
public:
     SpotExponentialOUVG(double sconst=1) { s_const = sconst; };
     SpotExponentialOUVG(double alpha, double sigma, double mu, double sconst, double m, double theta, double kappa);
     ~SpotExponentialOUVG(void);

// Copy
public:
    SpotExponentialOUVG(const SpotExponentialOUVG& spot);
    SpotExponentialOUVG& operator = (const SpotExponentialOUVG& spot);

public:
    virtual SpotExponentialOUVG* clone(void) const;

    //----------- returns kind of the spot -------------------------------------
    std::string identifier(void) const { return "SpotExponentialOUVG"; }

    virtual Model get_model(void) const { return EXP_OU_VG; }

    //
    const std::vector<std::vector<double> >& diffusion(void);
    const std::vector<double>& drift(void);

    //--------------------------------------------------------------------------
    virtual void set_vgBrownianDriftCoef_theta(const double& vgBrownianDriftCoef_theta) { this->vgBrownianDriftCoef_theta = vgBrownianDriftCoef_theta; };
    virtual void set_vgGammaShape_kappa(const double& vgGammaShape_kappa) { this->vgGammaShape_kappa = vgGammaShape_kappa; }; 
    virtual void set_vgAdditionalDrift_m(const double& vgAdditionalDrift_m) { this->vgAdditionalDrift_m = vgAdditionalDrift_m; };
    virtual void set_spot_scale(const double& spot_scale) { this->s_const = spot_scale; };

    //--------------------------------------------------------------------------
    virtual double get_vgBrownianDriftCoef_theta() const { return  vgBrownianDriftCoef_theta; };
    virtual double get_vgGammaShape_kappa() const { return vgGammaShape_kappa; };
    virtual double get_vgAdditionalDrift_m() const { return vgAdditionalDrift_m; };
    virtual double get_spot_scale() const { return s_const;};

    //-----------  function to simulate a spot ----------------------------------
    // function two simulate spot between two time
    virtual double simulate(const double &dt, const double &spotValue) const;

    virtual void print_model_param() const;
};

#endif // carbonquant_ExponentialOUVG_h
