/*****************************************************************************
 * FAMOSA
 *****************************************************************************
 * Optimizer
 * 2010-03-17
 * REGIS DUVIGNEAU 
 * 
 *****************************************************************************
 *               Copyright (C) 2008 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments : shape optimization methods         
 ****************************************************************************/

#ifndef carbonquant_OPTIMIZER_H
#define carbonquant_OPTIMIZER_H

#include "carbonquantOptimizerExport.h"

#include "MapOptimizer.h"

#include <iostream>
#include <fstream>
#include <vector>
#include <stdlib.h>

//
// class to define the optimizer 
//
class CARBONQUANTOPTIMIZER_EXPORT Optimizer
{
public:
    enum Type {
        ES,
        ES_CMA
    };

public:
  // constructor
  Optimizer(void);
  Optimizer(int objSize, double stopCrit, int maxIt);
  // destructor
  virtual ~Optimizer(void);

public:
  virtual void init();

  // virtual ask and tell functions
  virtual std::vector<std::vector<double> > ask() {return std::vector<std::vector<double> >();};
  virtual void tell(std::vector<std::vector<double> >& desVect, std::vector<std::vector<double> >& objfun) {};

  // function to set attibutes
  virtual void setStartPoint(const std::vector<double>& startPoint);
  void updateBestFit(const std::vector<double>& value);

  // function to get attributes
  int getEvalNumber() const;
  int getOptContinue() const;
  const std::vector<double>& getBestVect() const;

  // function to activate/desactivate writing
  void setOutputFiles(bool value);
  void setOutputScreen(bool value);

protected:

 // problem dimension
  int dim;
  // number of objective function
  int objdim;
  // number of evaluations
  int eval_number;
  // it counter
  int itcount;
  // stopping criterion
  double stop_crit;
  // max iteration number
  int max_it;
  // stopping flag
  bool optcontinue;

  // file writing
  bool write_files;
  // screen outputs
  bool write_screen;

  // best design vector
  std::vector<double> bestVect;

  // best value obtained
  std::vector<double> bestfit;

  // total number of evaluations
  int total_eval_number;

  // function to compute vector scalar product
  double scalar_product_vector(const std::vector<double>& vectA, const std::vector<double>& vectB);

  // function to compute matrix vector product
  std::vector<double> product_matrix_vector(const std::vector<std::vector<double> >& matA, const std::vector<double>& vectB);

  // function to compute matrix  product
  std::vector<std::vector<double> > product_matrix(const std::vector<std::vector<double> >& matA, const std::vector<std::vector<double> >& matB);

  // function to initialize identity matrix
  std::vector<std::vector<double> > initialize_Id_matrix(void);

  // function to initialize zeros matrix
  std::vector<std::vector<double> > initialize_zeros_matrix(void);

  // function to transpose matrix 
  std::vector<std::vector<double> > transpose_matrix(const std::vector<std::vector<double> >& matB);

  // function to display vector
  void display_vector(const std::vector<double>& vect, int N);

  // function to display rows matrix
  void display_matrix_rows(const std::vector<std::vector<double> >& matA, int Nl, int Nc);

  // function to display columns matrix
  void display_matrix_columns(const std::vector<std::vector<double> >& matA, int Nl, int Nc);

  // function to print data 
  void printDataBase(const std::vector<double>& vect, double func, int iter, bool iosapp);  
};

typedef Optimizer::Type OptimizerType;

#endif
