/*****************************************************************************
 * FAMOSA
 *****************************************************************************
 * Optimizer_ES
 * 2010-03-17
 * REGIS DUVIGNEAU 
 * 
 *****************************************************************************
 *               Copyright (C) 2008 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments : evolution strategy       
 ****************************************************************************/

# ifndef carbonquant_OPTIMIZER_ES_H
# define carbonquant_OPTIMIZER_ES_H

#include "carbonquantOptimizerExport.h"

#include "Optimizer.h"

//
// class for ES variables (mu+lambda)
//
class CARBONQUANTOPTIMIZER_EXPORT Optimizer_ES : public Optimizer
{
public:
  // constructors
  Optimizer_ES(double step, double stopCrit, int maxIt, int population_size);

  // destructor
  ~Optimizer_ES(void);

  // ask function
  std::vector<std::vector<double> > ask();
  // tell function
  void tell(std::vector<std::vector<double> >& desVect, std::vector<std::vector<double> >& objfun);

  virtual void setStartPoint(const std::vector<double>& startPoint);

  virtual void init();

private:
  // children population size
  int lambda;
  // parents population size
  int mu;
  // step size
  double step;

  // mean point
  std::vector<double> xbar;
  // mean deviation
  double sigmabar;
  // learning coefficient
  double tau;
  // deviation array
  std::vector<double> sigma;
};

//
// function for normal random distribution Box Muller's algo
//
double CARBONQUANTOPTIMIZER_EXPORT normalrand(void);

#endif
