/*****************************************************************************
 * FAMOSA
 *****************************************************************************
 * Optimizer_ES_CMA
 * 2010-07-12
 * LOUIS BLANCHARD 
 * 
 *****************************************************************************
 *               Copyright (C) 2008 INRIA Sophia-Antipolis
 *****************************************************************************
 * Comments : Evolution Strategy Covariance Matrix Adaptation       
 ****************************************************************************/

#ifndef carbonquant_OPTIMIZER_ES_CMA_H
#define carbonquant_OPTIMIZER_ES_CMA_H

#include "carbonquantOptimizerExport.h"

#include "Optimizer.h"

//
// class for ES CMA variables (mu and lambda)
//
class CARBONQUANTOPTIMIZER_EXPORT Optimizer_ES_CMA : public Optimizer
{
public:
  // constructor
  Optimizer_ES_CMA(double step, double stopCrit, int maxIt);
  // destructor
  ~Optimizer_ES_CMA(void);

  // ask function
  std::vector<std::vector<double> > ask();
  // tell function
  void tell(std::vector<std::vector<double> >& desVect, std::vector<std::vector<double> >& objfun);

  virtual void setStartPoint(const std::vector<double>& startPoint);

  virtual void init();

 private:
  // children population size
  int lambda;
  // parents population size
  int mu;

  // mean point
  std::vector<double> xbar;

  //--------------------------
  // covariance matrix
  std::vector<std::vector<double> > matC;
 // inverse sqrt covariance matrix
  std::vector<std::vector<double> > matC_invsqrt;
  // orthogonal matrix
  std::vector<std::vector<double> > matB;
  // diagonal matrix
  std::vector<double> matD;
  // evolution path : for update covariance matrix
  std::vector<double> pc;
  // coordinate wise standard deviation (step size)
  double step;
  double sigma;
  // conjugate evolution path for update sigma
  std::vector<double> ps;
  // weights array for recombination
  std::vector<double> w;
  // variance effectiveness of sum w[i]x[i]
  double mu_eff;
 // chiN expectation of ||N(0,I)|| == norm(randn(N,1))
  double chiN;
  // Heaviside function
  int hs;
  // test for Heaviside function
  double test_hs;
  // counters
  int eigeneval;
  // constante using update matrix to achieve 0(N^2)
  double const_update_mat;

  //-------- Adaptation Parameter --------
  // constant for  update covariance matrix
  double cc;
  double cs;
  double puiss_cs;
  double c1;
  double cmu;
  double ds;

  // --------function to update rank mu matrix -----------
  std::vector<std::vector<double> > rank_mu_update_matrix(const std::vector<std::vector<double> >& mat, const std::vector<double>& vect);
  //  function for normal random distribution
  std::vector<double> normalrandVect(void);

  // function to sort the lambda population 
  void sort_population(std::vector<std::vector<double> >& desVect, std::vector<std::vector<double> >& objfun);
  // function to display ant test DSYEV function 
  void display_test_DSYEV(const std::vector<std::vector<double> >& mat);
 // function to print Data
  void printDataOpt(bool iosapp);
  // function to print Data
  void printDataBase_ES_CMA(const std::vector<std::vector<double> >& desVect, const std::vector<std::vector<double> >& objfun);
};

#endif
