/*
  This file has my implementation of the LAPACK routine dsyev
  for C++.  This program solves for the eigenvalues and, if
  desired, the eigenvectors for a square symmetric matrix H.
  It assumes that the upper triangle of H is stored.

  There are two function calls defined in this header, of the
  forms

    void dsyev(double **H, int n, double *E)
    void dsyev(double **H, int n, double *E, double **Evecs)

    H: the n by n symmetric matrix that we are solving, with the
       upper triangle stored
    n: the order of the square matrix H
    E: an n-element array to hold the eigenvalues of H
    Evecs: an n by n matrix to hold the eigenvectors of H, if
           they are requested.

  The function call is defined twice, so that whether or not
  eigenvectors are called for the proper version is called.

  Scot Shaw
  30 August 1999
*/

# ifndef SCOT_SHAW_DSYEV_H
# define SCOT_SHAW_DSYEV_H


#include <math.h>

void Scot_Shaw_dsyev(double **H, int n, double *E);
void Scot_Shaw_dsyev(double **H, int n, double *E, double **Evecs);

double *Scot_Shaw_dsyev_ctof(double **in, int rows, int cols);
double **Scot_Shaw_dsyev_ftoc(double *in, int rows, int cols);
void Scot_Shaw_dsyev_ftoc2(double *in, double **out, int rows, int cols);
void Scot_Shaw_dsyev_sort(double *E, double **Evecs, int N);

#endif // SCOT_SHAW_DSYEV_H
