/* carbonquantPlotView.h --- 
 */

/* Commentary: 
 * 
 */

#ifndef CARBONQUANTPLOTVIEW_H
#define CARBONQUANTPLOTVIEW_H

#include "carbonquantPlotExport.h"

#include "carbonquantPlotCurve.h"

#include <dtkCore/dtkAbstractView.h>

#include <qwt_plot.h>
#include <qwt_scale_widget.h>
#include <qwt_color_map.h>

#include <QtCore>
#include <QtGui>

class carbonquantPlotHistogram;

class carbonquantPlotViewPrivate;

class CARBONQUANTPLOT_EXPORT carbonquantPlotView : public dtkAbstractView
{
    Q_OBJECT

public:
    enum Scale {
        Linear,
        Logarithmic
    };

    enum LegendPosition {
        RightLegend,
        BottomLegend,
        LeftLegend,
        TopLegend
    };

public:
     carbonquantPlotView(QWidget *parent = 0);
     carbonquantPlotView(const carbonquantPlotView&) {};
    ~carbonquantPlotView(void);

public:
    carbonquantPlotView& operator=(const carbonquantPlotView&) {};
    
    void clear(void);

    void setTitle(const QString&);
    void setTitleSize(const int&);
    QString title(void) const;
    int titleSize(void) const;

    void setAxisTitleX(const QString&);
    void setAxisTitleY(const QString&);
    void setAxisTitle(const QString&, const int& axisId = QwtPlot::yRight);

    QString axisTitleX(void) const;
    QString axisTitleY(void) const;

    void setAxisTitleSizeX(const int&);
    void setAxisTitleSizeY(const int&);
    void setAxisTitleSize(const int&, const int& axisId = QwtPlot::yRight);

    int axisTitleSize(const int& axisId = QwtPlot::xBottom) const;

    void setAxisScaleX(const double&, const double&);
    void setAxisScaleY(const double&, const double&);

    void setAxisScaleX(const QwtInterval&);
    void setAxisScaleY(const QwtInterval&);

    void setAxisScaleX(Scale);
    void setAxisScaleY(Scale);

    void setAxisScaleDrawDateX(void);

    void setAxisMaxMinorX(const int&);
    void setAxisMaxMinorY(const int&);
    void setAxisMaxMinor(const int&, const int& axisId = QwtPlot::yRight);

    void setAxisMaxMajorX(const int&);
    void setAxisMaxMajorY(const int&);
    void setAxisMaxMajor(const int&, const int& axisId = QwtPlot::yRight);

    void setAxisColorMap(const QwtInterval&, QwtColorMap*, const int& axisId = QwtPlot::yRight);
    void setAxisColorBarEnabled(const bool&, const int& axisId = QwtPlot::yRight);

    void setBackgroundColor(const QColor&);
    const QColor& backgroundColor(void) const;

    void setForegroundColor(const QColor&);
    const QColor& foregroundColor(void) const;

    void setStyleSheet(const QString&);

    void updateZoom(void);

    void updateAxes(void);
    void enableAxisYright(bool tf = true);

    void setAlphaCurveAreaColor(int);
    double alphaCurveAreaColor(void);

    void insertLegend(LegendPosition position = RightLegend);
    void setLegendPosition(LegendPosition);
    LegendPosition legendPosition(void) const;
    void setLegendSize(const int&);
    int legendSize(void) const;
    void setLegendVisible(bool);
    bool isLegendVisible(void) const;

    void setGridVisible(bool);
    bool isGridVisible(void) const;

    void setCurveStyle(carbonquantPlotCurve::Style);

    carbonquantPlotView& operator<<(carbonquantPlotCurve*);
    carbonquantPlotView& operator<<(carbonquantPlotHistogram*);

    QList<carbonquantPlotCurve *> plotCurveList(void) const;
    carbonquantPlotCurve* plotCurve(const QString& name = QString()) const;

    QWidget* widget(void);

    QString content(void) const;

public:
    virtual QString identifier(void) const;

    static bool registered(void);

protected:
    void onRenderer(const QString&);

    void updateColors(void);

    void fillCurveArea(void);

signals:
    void detach(void);

public slots:
    void update(void);

    void onExport(void);

protected slots:
    void showItem(QwtPlotItem*, bool);

private:
    friend class carbonquantPlotCurve;
    friend class carbonquantPlotHistogram;
    friend class carbonquantPlotRenderer;

private:
    carbonquantPlotViewPrivate *d;
};

CARBONQUANTPLOT_EXPORT dtkAbstractView *createPlotView(void);

#endif
