#ifndef carbonquant_random_h
#define carbonquant_random_h

#include "carbonquantRandomStuffExport.h"

class CARBONQUANTRANDOMSTUFF_EXPORT random
{
private:
    // Seed used for random number generation
    static int random_seed;

public:
    random(void) {};
    ~random(void) {};

    /** Generates the seed of the pseudo random generator.
     */
    static int set_seed(int=-1);

    /** Generates a uniform random or pseudo random value using SFMT.
     */
    static double rand_sfmt(double=0, double=1);

    /** Generate a Gaussian r.v. with (expectation = mu, variance = sigma^2) by Box-Muller method.
    */
    static double normal_distribution(double=0, double=1);

    /** Generate a Gaussian r.v. with (expectation = mu, variance = sigma^2) by the rejection  method.
     */
    static double normal_distribution_by_rejection(double=0, double=1);

    /** Generate a Gamma r.v. with expected value=k*theta and variance=k*theta^2.
     * shape < 1  use a version of the  Best/Ahrens/Dieter algorithm
     * shape > 1  use the additive property and the fact that G(1,1) = Exp(1)
     * references : wikipedia
     */
    static double gamma_distribution(double, double);

    /** Generate a Gamma r.v. with expected value=k*theta and variance=k*theta^2.
     */
    static double gamma_distribution_unknown(double, double);

    /** Generate a Gamma r.v. with expected value=k*theta and variance=k*theta^2.
     * shape < 1  use the Best/Ahrens/Dieter algorithm (cf Gentle Book)
     * shape > 1  use the Cheng/Feast algorithm cf Gentle Book)
     */
    static double gamma_distribution_gentle(double, double);
};

#endif // carbonquant_random_h
